#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses
import curses.panel
import os.path

import window
import events
import filelistwin
import playlistwin
import playerwin
import iteminfowin
import helpwin
import statusbar
import songdb
import players
import timer
import config

from helper import debug

# Uncomment the next line, if you want to experience a little bit with
# the number of bytecode instructions after which a context switch
# occurs.

# import sys; sys.setcheckinterval(250)

class mainwin(window.window):
    def __init__(self, screen, h, w, y, x):
        window.window.__init__(self, screen, h, w, y, x, None)
        self.screen = screen
        self.h = h
        self.w = w
        leftpanelw = int(w/2.2)
        rightpanelw = w-leftpanelw

        config.basedir = os.path.expanduser(config.basedir)
        config.songdbbase = os.path.expanduser(config.songdbbase)
        config.logfile = os.path.expanduser(config.logfile)

        self.channel = events.hub.newchannel()
        self.timer = timer.timer()

        self.songdb = songdb.songdb(self.channel, config.songdbbase)
        songdb.songautoregisterer(self.songdb, config.basedir).start()

        self.keybindings = config.keybindings["global"]

        self.statusbar = statusbar.statusbar(screen, h-1, w, self.channel)

        self.filelistwin = filelistwin.filelistwin(screen,
                                                   h-1, leftpanelw, 0, 0,
                                                   config.filelistwinhasscrollbar,
                                                   self.channel,
                                                   config.basedir,
                                                   self.songdb,
                                                   config.keybindings["filelist"],
                                                   )

        self.playlistwin = playlistwin.playlistwin(screen,
                                                   h-6-3-1, rightpanelw, 9, leftpanelw,
                                                   config.playlistwinhasscrollbar,
                                                   self.channel,
                                                   self.songdb,
                                                   config.logfile,
                                                   config.keybindings["playlist"])

        self.players = []
        for player in config.players:
            name = player["name"]
            options = player["options"]
            if name=="xmms":
                import players.xmmsplayer
                self.players.append(players.xmmsplayer.player(**options))
            elif name=="mpg123":
                import players.mpg123player
                self.players.append(players.mpg123player.player(**options))
            elif name=="hip":
                import players.hipplayer
                self.players.append(players.hipplayer.player(**options))
            elif name=="mad":
                import players.madplayer
                self.players.append(players.madplayer.player(**options))

        for player in self.players:
            player.start()

        self.playerwin = playerwin.playerwin(screen,
                                             3, rightpanelw, 0, leftpanelw,
                                             self.channel,
                                             self.players[0],
                                             config.keybindings)

        try:
            sp = self.players[1]
        except IndexError:
            sp = None
            
        self.iteminfowin = iteminfowin.iteminfowin(screen,
                                                   6, rightpanelw, 3, leftpanelw,
                                                   self.channel,
                                                   sp)

        self.helpwin = helpwin.helpwin(self.screen,
                                       self.h, self.w, 0, 0,
                                       self.channel,
                                       self.timer)
        
        if config.mixer is not None:
            import mixerwin
            mixerwidth = 36
            self.mixerwin = mixerwin.mixerwin(screen,
                                              3, mixerwidth,
                                              (self.h-3)/2, (self.w-mixerwidth)/2,
                                              self.channel,
                                              self.timer,
                                              config.mixer,
                                              config.keybindings["global"])
        else:
            del config.keybindings["global"]["volumeup"]
            del config.keybindings["global"]["volumedown"]
        
        self.channel.subscribe(events.keypressed, self.keypressed)
        self.channel.subscribe(events.activateplaylist, self.activateplaylist)
        self.channel.subscribe(events.activatefilelist, self.activatefilelist)

        events.hub.notify(events.activatefilelist())

        curses.curs_set(0)
        curses.meta(1)
        curses.raw()
        curses.halfdelay(1)

        while 1:
            try:
                key = self.screen.getch()
                
                if key==27:
                    # handle escape sequence (e.g. alt+key)
                    key = self.screen.getch()+1024
                    
                if key in self.keybindings["exit"]:
                    curses.halfdelay(5)
                    key = self.screen.getch()
                    curses.halfdelay(1)
                    if key in self.keybindings["exit"]:
                        break
                elif key!=-1:
                    events.hub.notify(events.keypressed(key))

                self.timer.process()
                self.channel.process()

                # update screen
                curses.panel.update_panels()
                curses.doupdate()

            except KeyboardInterrupt:
                pass
            
        for player in self.players:
            player.playerquit()

        self.songdb.close()

    def refresh(self):
        # refresh screen (i don't know any better way...)
        self.screen.clear()
        self.filelistwin.update()
        self.playlistwin.update()
        self.playerwin.update()
        self.iteminfowin.update()
        self.statusbar.update()

    # event handler

    def activateplaylist(self, event):
        self.playlistwin.top()

    def activatefilelist(self, event):
        self.filelistwin.top()

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["refresh"]:
            self.refresh()
        elif key in self.keybindings["playlistdeleteplayedsongs"]:
            events.hub.notify(events.playlistdeleteplayedsongs())
        elif key in self.keybindings["playlistclear"]:
            events.hub.notify(events.playlistclear())
        elif key in self.keybindings["playlistload"]:
            events.hub.notify(events.playlistload())
        elif key in self.keybindings["playlistsave"]:
            events.hub.notify(events.playlistsave())
        elif key in self.keybindings["showhelp"]:
            if self.filelistwin.hasfocus():
                context = "filelist"
            elif self.playlistwin.hasfocus():
                context = "playlist"
            else:
                context = None

            self.helpwin.showhelp(context)
        else:
            debug("unknown key: %d\n" % key)

def main(stdscr):
    mw = mainwin(stdscr, curses.LINES, curses.COLS, 0, 0)
        
curses.wrapper(main)
